@@ -707,30 +707,62 @@ static int ipkg_install_pending_cmd(ipkg_conf_t *conf, int argc, char **argv)
 
 static int ipkg_remove_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
-     int i;
+     int i,err,a,done;
      pkg_t *pkg;
-
+     pkg_t *pkg_to_remove;
+     pkg_vec_t *available;
+     char *pkg_name = NULL;
      global_conf = conf;
+     regex_t re;
      signal(SIGINT, sigint_handler);
 
-     pkg_info_preinstall_check(conf);
-     if (argc > 0) {
-	  for (i=0; i < argc; i++) {
-	       if (conf->restrict_to_default_dest) {
-		    pkg = pkg_hash_fetch_installed_by_name_dest(&conf->pkg_hash,
-								argv[i],
-								conf->default_dest);
-	       } else {
-		    pkg = pkg_hash_fetch_installed_by_name(&conf->pkg_hash, argv[i]);
-	       }
+// ENH: Add the "no pkg removed" just in case.
 
-	       if (pkg == NULL) {
-		    ipkg_message(conf, IPKG_ERROR,
-				 "Package %s is not installed.\n", argv[i]);
-		    continue;
-	       }
-	       ipkg_remove_pkg(conf, pkg);
-	  }
+    done = 0;
+
+     available = pkg_vec_alloc();
+     pkg_info_preinstall_check(conf);
+     if ( argc > 0 ) {
+        pkg_hash_fetch_all_installed(&conf->pkg_hash, available);
+        for (i=0; i < argc; i++) {
+           // pkg_name = argv[i];
+           pkg_name = malloc(strlen(argv[i])+2);
+           strcpy(pkg_name,argv[i]);
+           err = ipkg_regcomp(&re, pkg_name);
+           if (err) {
+               return err;
+           }
+           for (a=0; a < available->len; a++) {
+               pkg = available->pkgs[a];
+               if (pkg_name && xregexec(&re, pkg->name, 0, 0, 0)) {
+                  continue;
+               }
+               if (conf->restrict_to_default_dest) {
+	            pkg_to_remove = pkg_hash_fetch_installed_by_name_dest(&conf->pkg_hash,
+							        pkg->name,
+							        conf->default_dest);
+               } else {
+	            pkg_to_remove = pkg_hash_fetch_installed_by_name(&conf->pkg_hash, pkg->name );
+               }
+        
+               if (pkg == NULL) {
+	            ipkg_message(conf, IPKG_ERROR, "Package %s is not installed.\n", pkg->name);
+	            continue;
+               }
+               if (pkg->state_want == SW_DEINSTALL) {    // Added the control, so every already removed package could be skipped
+	            ipkg_message(conf, IPKG_ERROR, "Package %s already removed.\n", pkg->name);
+	            continue;
+               }
+               ipkg_remove_pkg(conf, pkg_to_remove);
+               done = 1;
+               // a=available->len;  // Let's go out from here. We should remove the next one.
+               // pkg_hash_fetch_all_installed(&conf->pkg_hash, available);  // This could not be needed, but just in case....
+           }
+           if (pkg_name)
+              xregfree(&re);
+           free (pkg_name);
+        }
+        pkg_vec_free(available);
      } else {
 	  pkg_vec_t *installed_pkgs = pkg_vec_alloc();
 	  int i;
@@ -766,12 +798,16 @@ static int ipkg_remove_cmd(ipkg_conf_t *conf, int argc, char **argv)
 			 removed++;
 			 ipkg_message(conf, IPKG_NOTICE, "Removing non-user leaf package %s\n");
 			 ipkg_remove_pkg(conf, pkg);
+                         done = 1;
 		    }
 	       }
 	  } while (removed);
 	  pkg_vec_free(installed_pkgs);
      }
 
+     if ( done == 0 ) 
+        ipkg_message(conf, IPKG_NOTICE, "No packages removed.\n");
+
      write_status_files_if_changed(conf);
      return 0;
 }
@@ -1260,3 +1296,4 @@ static int ipkg_print_architecture_cmd(ipkg_conf_t *conf, int argc, char **argv)
      }
      return 0;
 }
+
