@@ -58,7 +58,6 @@ void pkg_hash_deinit(hash_table_t *hash)
   hash_table_deinit(hash);
 }
 
-
 
 int pkg_hash_add_from_file(ipkg_conf_t *conf, const char *file_name,
 			   pkg_src_t *src, pkg_dest_t *dest, int is_status_file)
@@ -128,7 +127,8 @@ pkg_t *pkg_hash_fetch_best_installation_candidate(ipkg_conf_t *conf, abstract_pk
      pkg_t *latest_matching = NULL;
      pkg_t *held_pkg = NULL;
 
-     if (apkg == NULL || apkg->provided_by == NULL || (apkg->provided_by->len == 0))
+     if (matching_apkgs == NULL || providers == NULL || 
+         apkg == NULL || apkg->provided_by == NULL || (apkg->provided_by->len == 0))
 	  return NULL;
 
      ipkg_message(conf, IPKG_DEBUG, "best installation candidate for %s\n", apkg->name);
@@ -475,8 +475,10 @@ pkg_t *hash_insert_pkg(hash_table_t *hash, pkg_t *pkg, int set_status)
 
      arch_priority = pkg->arch_priority;
 
-     buildDepends(hash, pkg);
-
+     if (buildDepends(hash, pkg)<0){
+        fprintf(stderr, "%s : This should never happen. Report this Bug in bugzilla please \n ",__FUNCTION__);
+        return NULL;
+     }
      ab_pkg = ensure_abstract_pkg_by_name(hash, pkg->name);
 
      if (set_status) {
@@ -494,10 +496,19 @@ pkg_t *hash_insert_pkg(hash_table_t *hash, pkg_t *pkg, int set_status)
      pkg = pkg_vec_insert_merge(ab_pkg->pkgs, pkg, set_status);
      pkg->parent = ab_pkg;
 
-     buildProvides(hash, ab_pkg, pkg);
+     if (buildProvides(hash, ab_pkg, pkg)<0){
+        fprintf(stderr, "%s : This should never happen. Report this Bug in bugzilla please \n ",__FUNCTION__);
+        return NULL;
+     }
      /* need to build the conflicts graph before replaces for correct calculation of replaced_by relation */
-     buildConflicts(hash, ab_pkg, pkg);
-     buildReplaces(hash, ab_pkg, pkg);
+     if (buildConflicts(hash, ab_pkg, pkg)<0){
+        fprintf(stderr, "%s : This should never happen. Report this Bug in bugzilla please \n ",__FUNCTION__);
+        return NULL;
+     }
+     if (buildReplaces(hash, ab_pkg, pkg)<0) {
+        fprintf(stderr, "%s : This should never happen. Report this Bug in bugzilla please \n ",__FUNCTION__);
+        return NULL;
+     }
     
      buildDependedUponBy(pkg, ab_pkg);
      return pkg;
